@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-document.h,v 1.20 2005/04/05 00:40:57 krh Exp $
+ *  $Id: ev-document.h,v 1.24 2005/04/22 03:39:51 jrb Exp $
  */
 
 #ifndef EV_DOCUMENT_H
@@ -27,6 +27,7 @@
 #include <gdk/gdk.h>
 
 #include "ev-link.h"
+#include "ev-document-info.h"
 
 G_BEGIN_DECLS
 
@@ -46,7 +47,8 @@ typedef struct _EvPageCacheClass  EvPageCacheClass;
 
 
 #define EV_DOCUMENT_ERROR ev_document_error_quark ()
-#define EV_DOC_MUTEX (ev_document_get_doc_mutex ())
+#define EV_DOC_MUTEX_LOCK (ev_document_doc_mutex_lock ())
+#define EV_DOC_MUTEX_UNLOCK (ev_document_doc_mutex_unlock ())
 
 typedef enum
 {
@@ -79,6 +81,7 @@ struct _EvDocumentIface
 					 double       *height);
 	char	  * (* get_page_label)  (EvDocument   *document,
 					 int           page);
+	gboolean    (* can_get_text)    (EvDocument   *document);
 	char	  * (* get_text)	(EvDocument   *document,
 					 int           page,
 					 EvRectangle  *rect);
@@ -87,37 +90,38 @@ struct _EvDocumentIface
 	GdkPixbuf * (* render_pixbuf)   (EvDocument   *document,
 					 int           page,
 					 double        scale);
+	EvDocumentInfo *(* get_info)    (EvDocument   *document);
 };
 
-GType        ev_document_get_type       (void);
-GQuark       ev_document_error_quark    (void);
-EvPageCache *ev_document_get_page_cache (EvDocument *document);
-GMutex      *ev_document_get_doc_mutex  (void);
-
-
-gboolean   ev_document_load          (EvDocument    *document,
-				      const char    *uri,
-				      GError       **error);
-gboolean   ev_document_save          (EvDocument    *document,
-				      const char    *uri,
-				      GError       **error);
-char      *ev_document_get_title     (EvDocument    *document);
-int        ev_document_get_n_pages   (EvDocument    *document);
-void       ev_document_get_page_size (EvDocument    *document,
-				      int            page,
-				      double        *width,
-				      double        *height);
-char      *ev_document_get_page_label(EvDocument    *document,
-				      int            page);
-char      *ev_document_get_text      (EvDocument    *document,
-				      int            page,
-				      EvRectangle   *rect);
-GList     *ev_document_get_links     (EvDocument    *document,
-				      int            page);
-GdkPixbuf *ev_document_render_pixbuf (EvDocument    *document,
-				      int            page,
-				      double         scale);
-
+GType		ev_document_get_type       (void);
+GQuark		ev_document_error_quark    (void);
+EvPageCache    *ev_document_get_page_cache (EvDocument     *document);
+GMutex	       *ev_document_get_doc_mutex  (void);
+void            ev_document_doc_mutex_lock (void);
+void            ev_document_doc_mutex_unlock (void);
+EvDocumentInfo *ev_document_get_info	   (EvDocument     *document);
+gboolean	ev_document_load	   (EvDocument     *document,
+					    const char     *uri,
+					    GError        **error);
+gboolean	ev_document_save	   (EvDocument     *document,
+					    const char     *uri,
+					    GError        **error);
+int		ev_document_get_n_pages	   (EvDocument     *document);
+void		ev_document_get_page_size  (EvDocument     *document,
+					    int             page,
+					    double         *width,
+					    double         *height);
+char	       *ev_document_get_page_label (EvDocument     *document,
+					    int             page);
+gboolean	ev_document_can_get_text   (EvDocument     *document);
+char	       *ev_document_get_text       (EvDocument     *document,
+					    int             page,
+					    EvRectangle    *rect);
+GList	       *ev_document_get_links      (EvDocument     *document,
+				            int             page);
+GdkPixbuf      *ev_document_render_pixbuf  (EvDocument     *document,
+				      	    int             page,
+					    double          scale);
 
 G_END_DECLS
 
