@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: main.c,v 1.9 2005/03/23 11:07:32 marco Exp $
+ *  $Id: main.c,v 1.11 2005/04/09 11:26:38 chpe Exp $
  */
 
 #include "config.h"
@@ -26,6 +26,7 @@
 #include <gtk/gtkmain.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-app-helper.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "ev-stock-icons.h"
@@ -80,7 +81,7 @@ main (int argc, char *argv[])
                                       LIBGNOMEUI_MODULE, argc, argv,
                                       GNOME_PARAM_POPT_TABLE, popt_options,
                                       GNOME_PARAM_HUMAN_READABLE_NAME, _("Evince"),
-				      GNOME_PARAM_APP_DATADIR, DATADIR,
+				      GNOME_PARAM_APP_DATADIR, GNOMEDATADIR,
                                       NULL);
 
 	ev_job_queue_init ();
@@ -99,6 +100,7 @@ main (int argc, char *argv[])
 
 	gtk_main ();
 
+	gnome_accelerators_sync ();
 	poptFreeContext (context);
 
 	return 0;
