@@ -16,14 +16,13 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <ev-poppler.h>
-
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 
 #include <ev-document.h>
+#include <ev-document-types.h>
 #include <ev-document-thumbnails.h>
 
 #include <string.h>
@@ -35,18 +34,20 @@ evince_thumbnail_pngenc_get (const char *uri, const char *thumbnail, int size)
 {
 	EvDocument *document = NULL;
 	char *mime_type;
-	GError *error;
+	GError *error = NULL;
 	GdkPixbuf *pixbuf;
+	GType document_type;
 
 	mime_type = gnome_vfs_get_mime_type (uri);
 	if (mime_type == NULL)
 		return FALSE;
 
-	if (!strcmp (mime_type, "application/pdf"))
-		document = g_object_new (PDF_TYPE_DOCUMENT, NULL);
-	else
+	document_type = ev_document_type_lookup (mime_type);
+	if (document_type==G_TYPE_INVALID)
 		return FALSE;
 
+	document = g_object_new (document_type, NULL);
+
 	if (!ev_document_load (document, uri, &error)) {
 		if (error->domain == EV_DOCUMENT_ERROR &&
             	    error->code == EV_DOCUMENT_ERROR_ENCRYPTED) {
@@ -56,30 +57,44 @@ evince_thumbnail_pngenc_get (const char *uri, const char *thumbnail, int size)
 		return FALSE;
 	}
 
+	if (!EV_IS_DOCUMENT_THUMBNAILS (document)) {
+		return FALSE;
+	}
+
 	pixbuf = ev_document_thumbnails_get_thumbnail
-			(EV_DOCUMENT_THUMBNAILS (document), 1, size, FALSE);
+			(EV_DOCUMENT_THUMBNAILS (document), 0, size, FALSE);
 	
 	if (pixbuf != NULL) {
-		GdkPixbuf *pdflogo;
+		const char *overlaid_icon_name = NULL;
 
-		pdflogo = gdk_pixbuf_new_from_file (DATADIR"/pdf-icon.png", NULL);
-		if (pdflogo != NULL) {
-			int delta_height, delta_width;
+		if (strcmp(mime_type,"application/pdf")==0) {
+			overlaid_icon_name = "pdf-icon.png";
+		}
 
-			delta_width = gdk_pixbuf_get_width (pixbuf) -
-			              gdk_pixbuf_get_width (pdflogo);
-			delta_height = gdk_pixbuf_get_height (pixbuf) -
-			               gdk_pixbuf_get_height (pdflogo);
+		if (overlaid_icon_name) {
+			GdkPixbuf *overlaid_pixbuf;
 
-			gdk_pixbuf_composite (pdflogo, pixbuf,
-					      delta_width, delta_height,
-					      gdk_pixbuf_get_width (pdflogo),
-					      gdk_pixbuf_get_height (pdflogo),
-					      delta_width, delta_height,
-					      1, 1,
-					      GDK_INTERP_NEAREST, 100);
-
-			gdk_pixbuf_unref  (pdflogo);
+			gchar *overlaid_icon_path = g_strdup_printf ("%s/%s", DATADIR, overlaid_icon_name);
+			overlaid_pixbuf = gdk_pixbuf_new_from_file (overlaid_icon_path, NULL);
+			g_free (overlaid_icon_path);
+			if (overlaid_pixbuf != NULL) {
+				int delta_height, delta_width;
+				
+				delta_width = gdk_pixbuf_get_width (pixbuf) -
+					gdk_pixbuf_get_width (overlaid_pixbuf);
+				delta_height = gdk_pixbuf_get_height (pixbuf) -
+					gdk_pixbuf_get_height (overlaid_pixbuf);
+				
+				gdk_pixbuf_composite (overlaid_pixbuf, pixbuf,
+						      delta_width, delta_height,
+						      gdk_pixbuf_get_width (overlaid_pixbuf),
+						      gdk_pixbuf_get_height (overlaid_pixbuf),
+						      delta_width, delta_height,
+						      1, 1,
+						      GDK_INTERP_NEAREST, 100);
+				
+				gdk_pixbuf_unref  (overlaid_pixbuf);
+			}
 		}
 		if (gdk_pixbuf_save (pixbuf, thumbnail, "png", NULL, NULL)) {
 			gdk_pixbuf_unref  (pixbuf);
